package handler

import (
	"errors"
	"net/http"
	"net/http/httptest"
	"testing"

	"gitea.michaelthomson.dev/mthomson/habits/internal/todo/service"
)

type MockTodoDeleter struct {
	DeleteTodoFunc func(id int64) error
}

func (tg *MockTodoDeleter) DeleteTodo(id int64) error {
	return tg.DeleteTodoFunc(id)
}

func TestDeleteTodo(t *testing.T) {
	t.Run("deletes existing todo", func(t *testing.T) {
		service := MockTodoDeleter{
			DeleteTodoFunc: func(id int64) error {
				return nil
			},
		}

		handler := HandleTodoDelete(&service)

		req := httptest.NewRequest(http.MethodDelete, "/todo/1", nil)
		res := httptest.NewRecorder()
		req.SetPathValue("id", "1")

		handler(res, req)

		if res.Code != http.StatusNoContent {
			t.Errorf("did not get correct status, got %d, want %d", res.Code, http.StatusOK)
		}
	})

	t.Run("returns 400 with bad id", func(t *testing.T) {
		handler := HandleTodoDelete(nil)

		req := httptest.NewRequest(http.MethodDelete, "/todo/hello", nil)
		res := httptest.NewRecorder()
		req.SetPathValue("id", "hello")

		handler(res, req)

		if res.Code != http.StatusBadRequest {
			t.Errorf("did not get correct status, got %d, want %d", res.Code, http.StatusOK)
		}
	})

	t.Run("returns 404 for not found todo", func(t *testing.T) {
		service := MockTodoDeleter{
			DeleteTodoFunc: func(id int64) error {
				return service.ErrNotFound
			},
		}

		handler := HandleTodoDelete(&service)

		req := httptest.NewRequest(http.MethodDelete, "/todo/1", nil)
		res := httptest.NewRecorder()
		req.SetPathValue("id", "1")

		handler(res, req)

		if res.Code != http.StatusNotFound {
			t.Errorf("did not get correct status, got %d, want %d", res.Code, http.StatusOK)
		}
	})

	t.Run("returns 500 for arbitrary errors", func(t *testing.T) {
		service := MockTodoDeleter{
			DeleteTodoFunc: func(id int64) error {
				return errors.New("foo bar")
			},
		}

		handler := HandleTodoDelete(&service)

		req := httptest.NewRequest(http.MethodDelete, "/todo/1", nil)
		res := httptest.NewRecorder()
		req.SetPathValue("id", "1")

		handler(res, req)

		if res.Code != http.StatusInternalServerError {
			t.Errorf("did not get correct status, got %d, want %d", res.Code, http.StatusOK)
		}
	})

}
