package handler

import (
	"encoding/json"
	"errors"
	"fmt"
	"net/http"
	"net/http/httptest"
	"reflect"
	"testing"

	"gitea.michaelthomson.dev/mthomson/habits/internal/todo/service"
)

type MockTodoGetter struct {
	GetTodoFunc func(id int64) (service.Todo, error)
}

func (tg *MockTodoGetter) GetTodo(id int64) (service.Todo, error) {
	return tg.GetTodoFunc(id)
}

func TestGetTodo(t *testing.T) {
	t.Run("gets existing todo", func(t *testing.T) {
		todo := service.Todo{Id: 1, Name: "clean dishes", Done: false}
		wantedTodo := GetTodoResponse{Id: 1, Name: "clean dishes", Done: false}

		service := MockTodoGetter{
			GetTodoFunc: func(id int64) (service.Todo, error) {
				return todo, nil
			},
		}

		handler := HandleTodoGet(&service)

		req := httptest.NewRequest(http.MethodGet, fmt.Sprintf("/todo/%d", todo.Id), nil)
		res := httptest.NewRecorder()
		req.SetPathValue("id", "1")

		handler(res, req)

		if res.Code != http.StatusOK {
			t.Errorf("did not get correct status, got %d, want %d", res.Code, http.StatusOK)
		}

		var got GetTodoResponse
		err := json.NewDecoder(res.Body).Decode(&got)

		if err != nil {
			t.Fatalf("Unable to parse response from server %q into GetTodoResponse: %v", res.Body, err)
		}

		if !reflect.DeepEqual(got, wantedTodo) {
			t.Errorf("got %+v, want %+v", got, wantedTodo)
		}
	})

	t.Run("returns 400 with bad id", func(t *testing.T) {
		handler := HandleTodoGet(nil)

		req := httptest.NewRequest(http.MethodGet, "/todo/hello", nil)
		res := httptest.NewRecorder()
		req.SetPathValue("id", "hello")

		handler(res, req)

		if res.Code != http.StatusBadRequest {
			t.Errorf("did not get correct status, got %d, want %d", res.Code, http.StatusOK)
		}
	})

	t.Run("returns 404 for not found todo", func(t *testing.T) {
		service := MockTodoGetter{
			GetTodoFunc: func(id int64) (service.Todo, error) {
				return service.Todo{}, service.ErrNotFound
			},
		}

		handler := HandleTodoGet(&service)

		req := httptest.NewRequest(http.MethodGet, "/todo/1", nil)
		res := httptest.NewRecorder()
		req.SetPathValue("id", "1")

		handler(res, req)

		if res.Code != http.StatusNotFound {
			t.Errorf("did not get correct status, got %d, want %d", res.Code, http.StatusOK)
		}
	})

	t.Run("returns 500 for arbitrary errors", func(t *testing.T) {
		service := MockTodoGetter{
			GetTodoFunc: func(id int64) (service.Todo, error) {
				return service.Todo{}, errors.New("foo bar")
			},
		}

		handler := HandleTodoGet(&service)

		req := httptest.NewRequest(http.MethodGet, "/todo/1", nil)
		res := httptest.NewRecorder()
		req.SetPathValue("id", "1")

		handler(res, req)

		if res.Code != http.StatusInternalServerError {
			t.Errorf("did not get correct status, got %d, want %d", res.Code, http.StatusOK)
		}
	})

}
