package service

import (
	"errors"
	"log"

	"gitea.michaelthomson.dev/mthomson/habits/internal/todo/repository"
)

var (
	ErrNotFound error = errors.New("Todo cannot be found")
)

type Todo struct {
	Id   int64
	Name string
	Done bool
}

func NewTodo(name string, done bool) Todo {
	return Todo{Name: name, Done: done}
}

func TodoFromTodoRow(todoRow repository.TodoRow) Todo {
	return Todo{Id: todoRow.Id, Name: todoRow.Name, Done: todoRow.Done}
}

func TodoRowFromTodo(todo Todo) repository.TodoRow {
	return repository.TodoRow{Id: todo.Id, Name: todo.Name, Done: todo.Done}
}

func (t Todo) Equal(todo Todo) bool {
	return t.Id == todo.Id && t.Name == todo.Name && t.Done == todo.Done
}

type TodoRepository interface {
	Create(todo repository.TodoRow) (repository.TodoRow, error)
	GetById(id int64) (repository.TodoRow, error)
	Update(todo repository.TodoRow) error
	Delete(id int64) error
}

type TodoService struct {
	repo TodoRepository
}

func NewTodoService(todoRepo TodoRepository) *TodoService {
	return &TodoService{todoRepo}
}

func (s *TodoService) GetTodo(id int64) (Todo, error) {
	todo, err := s.repo.GetById(id)

	if err != nil {
		if err == repository.ErrNotFound {
			return Todo{}, ErrNotFound
		}

		return Todo{}, err
	}

	return TodoFromTodoRow(todo), err
}

func (s *TodoService) CreateTodo(todo Todo) (Todo, error) {
	todoRow := TodoRowFromTodo(todo)

	newTodoRow, err := s.repo.Create(todoRow)

	if err != nil {
		log.Print(err)
		return Todo{}, err
	}

	return TodoFromTodoRow(newTodoRow), err
}

func (s *TodoService) DeleteTodo(id int64) error {
	err := s.repo.Delete(id)

	if err == repository.ErrNotFound {
		return ErrNotFound
	}

	return err
}

func (s *TodoService) UpdateTodo(todo Todo) error {
	todoRow := TodoRowFromTodo(todo)

	err := s.repo.Update(todoRow)

	if err == repository.ErrNotFound {
		return ErrNotFound
	}

	return err
}
